/////////////////////////////////////////////////////////////////////
//
// ROUTER2.CPP
//
// Overview:
// 
// This program is a dummy program that simply does a get then put on
// an incoming message.
// There are no parameters.  The output queue is kept open.
//
// Sam Rogers 05/07/99
//
/////////////////////////////////////////////////////////////////////
//*******************************************************************
// Includes
//*******************************************************************
extern"C" 
{
#include <stdio.h>
#include <stdlib.h>	
}

#include <iostream.h>

// Includes for MQSeries C++
#include <imqi.hpp>


//*******************************************************************
// Constants
//*******************************************************************
// Environment
#define QMgr "PERFORMANCE"
#define InputQ "RULESIN"
#define OutputQ "OUTPUT"

//*******************************************************************
// Function Prototypes
//*******************************************************************

// Forwards supplied message to output queue
void ForwardMsg(ImqMessage msg, ImqQueue queue);

/////////////////////////////////////////////////////////////////////
// Start main program
/////////////////////////////////////////////////////////////////////
int main()
{

	// Objects needed
	ImqQueueManager mgr;				// Queue manager 
    ImqQueue queue;						// Input queue
	ImqQueue Oqueue;					// Output queue
	ImqMessage msg;						// Message
    ImqGetMessageOptions gmo;           // Get message options 
	ImqPutMessageOptions pmo;			// Put message options


 
	// Other
	bool Looping=true;					// Get loop control

	cout << "Starting Router program" << endl;

/////////////////////////////////////////////////////////////////////
// Setup and connect to MQIntegrator queue manager
/////////////////////////////////////////////////////////////////////
	
	mgr.setName(QMgr);
	
	if ( ! mgr.connect( ) ) {
		
		// stop if it failed 
		cerr << "Connect to queue manager ended with reason code "
             << (int)mgr.reasonCode( ) << "- program ending" << endl;
		exit(0);
	}

/////////////////////////////////////////////////////////////////////
// Setup and open output queue
/////////////////////////////////////////////////////////////////////
	Oqueue.setName(OutputQ);

	// Associate with qmgr
	Oqueue.setConnectionReference( mgr );

	// Open queue
	Oqueue.setOpenOptions( MQOO_OUTPUT ); // open queue for output        
	Oqueue.open();
	
	// Report reason, if any; stop if failed
	if ( Oqueue.reasonCode( ) ) {
		printf( "ImqQueue::open ended with reason code %d\n",
            (int)Oqueue.reasonCode( ) );
	}
	
	if ( Oqueue.completionCode( ) == MQCC_FAILED ) {
		printf( "Unable to open queue for output\n" );
		exit(0);
	}

	
/////////////////////////////////////////////////////////////////////
// Setup and open Router input queue
/////////////////////////////////////////////////////////////////////

	// Set name
	queue.setName(InputQ);

	// Associate with qmgr
	queue.setConnectionReference( mgr );

	// Open
	queue.setOpenOptions( MQOO_INPUT_AS_Q_DEF ); // open queue for input        
	queue.open();
	
	// Report reason, if any; stop if failed
	if ( queue.reasonCode( ) ) {
		printf( "ImqQueue::open ended with reason code %d\n",
            (int)queue.reasonCode( ) );
	}
	
	if ( queue.completionCode( ) == MQCC_FAILED ) {
		printf( "Unable to open queue for input\n" );
	}

/////////////////////////////////////////////////////////////////////
// Wait unlimited on the input queue
/////////////////////////////////////////////////////////////////////

	// Set options to unlimited wait
	gmo.setOptions( MQGMO_WAIT | MQGMO_FAIL_IF_QUIESCING | MQGMO_SYNCPOINT);
	pmo.setOptions(MQPMO_SYNCPOINT);
	gmo.setWaitInterval( MQWI_UNLIMITED );
	
	// Get messages and display
	while(Looping) {
		
		// Reset IDs
		msg.setMessageId( );
		msg.setCorrelationId( );
		
		// Get a message
		if ( queue.get( msg, gmo ) ) {
			ForwardMsg(msg, Oqueue);
		}
	
		// Commit 
	if(!mgr.commit()) {
		cerr << "Commit failed"
			 << endl;
		exit(0);
	}

	}

	
/////////////////////////////////////////////////////////////////////
// Close input queue
/////////////////////////////////////////////////////////////////////
	if ( ! queue.close( ) ) {
		
		// Report reason, if any     
		printf( "ImqQueue::close ended with reason code %d\n",
            (int)queue.reasonCode( ) );
	}

/////////////////////////////////////////////////////////////////////
// Close output queue
/////////////////////////////////////////////////////////////////////
	if ( ! Oqueue.close( ) ) {
		
		// Report reason, if any     
		printf( "ImqQueue::close ended with reason code %d\n",
            (int)Oqueue.reasonCode( ) );
	}
	
/////////////////////////////////////////////////////////////////////
// Disconnect from queue manager
/////////////////////////////////////////////////////////////////////
	if ( ! mgr.disconnect( ) ) {
		
		// Report reason, if any     
		printf( "ImqQueueManager::disconnect ended with reason code %d\n",
            (int)mgr.reasonCode( ) );
	}

/////////////////////////////////////////////////////////////////////
// END
/////////////////////////////////////////////////////////////////////
	cout << "Ending Router program..." << endl;
	return(1);
}

/////////////////////////////////////////////////////////////////////
// Function definitions
/////////////////////////////////////////////////////////////////////

void ForwardMsg(ImqMessage msg, ImqQueue queue) 
{
	
/////////////////////////////////////////////////////////////////////
// Put msg to queue
/////////////////////////////////////////////////////////////////////
	
	if ( ! queue.put( msg ) ) {
		
        // Report reason, if any 
        printf( "ImqQueue::put ended with reason code %d\n",
			(int)queue.reasonCode( ) );
	}


}
